function [HammingLoss,RankingLoss,OneError,Coverage,Average_Precision,Outputs,Pre_each_label,Pre_Labels]=KNN(train_data,train_target,test_data,test_target,Num)
%MLKNN_test tests a multi-label k-nearest neighbor classifier.
%
%    Syntax
%
%       [HammingLoss,RankingLoss,OneError,Coverage,Average_Precision,Outputs,Pre_Labels]=MLKNN_test(train_data,train_target,test_data,test_target,Num,Prior,PriorN,Cond,CondN)
%
%    Description
%
%       KNN takes,
%           train_data       - An M1xN array, the ith instance of training instance is stored in train_data(i,:)
%           train_target     - A QxM1 array, if the ith training instance belongs to the jth class, then train_target(j,i) equals +1, otherwise train_target(j,i) equals -1
%           test_data        - An M2xN array, the ith instance of testing instance is stored in test_data(i,:)
%           test_target      - A QxM2 array, if the ith testing instance belongs to the jth class, test_target(j,i) equals +1, otherwise test_target(j,i) equals -1
%           Num              - Number of neighbors used in the k-nearest neighbor algorithm
%      and returns,
%           HammingLoss      - The hamming loss on testing data
%           RankingLoss      - The ranking loss on testing data
%           OneError         - The one-error on testing data as
%           Coverage         - The coverage on testing data as
%           Average_Precision- The average precision on testing data
%           Outputs          - A QxM2 array, the probability of the ith testing instance belonging to the jCth class is stored in Outputs(j,i)
%           Pre_Labels       - A QxM2 array, if the ith testing instance belongs to the jth class, then Pre_Labels(j,i) is +1, otherwise Pre_Labels(j,i) is -1
%           Pre_each_label   - average precision of each label


[num_class,num_training]=size(train_target);
[num_class,num_testing]=size(test_target);


    
    
    %Computing distances between training instances and testing instances
    dist_matrix=zeros(num_class,num_training);
    for i=1:num_testing

        vector1=test_data(i,:);
        for j=1:num_training
            vector2=train_data(j,:);
            dist_matrix(i,j)=sqrt(sum((vector1-vector2).^2));
        end
    end
    

%Find neighbors of each testing instance
%    Neighbors=cell(num_testing,1); %Neighbors{i,1} stores the Num neighbors of the ith testing instance
    Pre_Labels=zeros(num_class,num_testing);
    temp=zeros(num_class,num_testing);
    for multi_labels=1:num_class     
         for i=1:num_testing
             [useless,index]=sort(dist_matrix(i,:));
  
              for j=1:Num
                 temp(multi_labels,i)=temp(multi_labels,i)+train_target(multi_labels,index(j));
              end
               if temp(multi_labels,i)>=0                              
                 Pre_Labels(multi_labels,i)=1;
               else
                 Pre_Labels(multi_labels,i)=-1; 
               end   
         end
    end




Outputs=(temp+Num)./(Num*2);
    

s=zeros(1,num_class);
for a=1:num_class
    for m=1:num_testing
        if Pre_Labels(a,m)==test_target(a,m);
            s(1,a)=s(1,a)+1;
        end
    end
    s(1,a)=s(1,a)/num_testing;
end

Pre_each_label=s;

    HammingLoss=Hamming_loss(Pre_Labels,test_target);
    
    RankingLoss=Ranking_loss(Outputs,test_target);
    OneError=One_error(Outputs,test_target);
    Coverage=coverage(Outputs,test_target);
    Average_Precision=Average_precision(Outputs,test_target);
    
    
    
        
        
        
        
        
        
        